const router = require('express').Router();
const {ensureJSON} = require('../helpers/middleware');
const ejs = require('ejs');
const formidable = require('formidable');
const config = require('../config');
const latex = require('node-latex');
const latexescape = require('../helpers/latexescape');
const fs = require('fs');
const path = require('path');

const registered_templates = [];

router.get('/:name', (req, res, next) => {
    try{
        const template_name = req.params['name'].toLowerCase().replace(/[^a-z]+/g, '');
        const template_conf = require(`../templates/${template_name}/conf.js`);

        if(!registered_templates.includes(template_name)){
            registered_templates.push(template_name);
        }

        res.render(`${template_name}/formular.ejs`, {template: template_conf, file_reset_timer: config.autodelete_uploads_interval});
    }
    catch(e){
        res.statusCode = 404;
        return next({
            name: 'Kunne ikke finde pdf templatet',
            message: ''
        });
    }
});


router.post('/upload/image', (req, res, _next) => {
    const form = new formidable.IncomingForm();
    form.maxFields = 0;
    form.maxFileSize = 2 * 1024 * 1024;
    form.type = true;

    let got_file = false;
    let file_id;
    let file_path;
    let errored = false;

    form.parse(req)

    form.on('fileBegin', (_name, file) => {
        if(got_file) return;
        got_file = true;
        const fileType = file.mimetype?.split?.('/')?.pop?.()?.toLowerCase?.();
        if(!['png', 'jpg', 'jpeg'].includes(fileType)){
            res.statusCode = 400;
            form.emit('error', new Error('Ugyldig filtype'));
            return;
        }

        const file_id = file.newFilename;
        file.name = file_id + '.' + fileType;
        file.filepath = config.tmp_path + file_id + '.' + fileType;
    });

    form.on('file', (_name, file) => {
        file_id = file.name;
        file_path = file.filepath;
    });

    form.on('end', () => {
        if(errored){
            if(file_path){
                fs.unlink(file_path, () => {});
            }
            return;
        }

        if(!file_id) {
            res.statusCode = 400;
            return res.json({error: 'Du mangler at vælge en fil til upload'});
        }
        else{
            autodelete(file_id);
            if (!res.headersSent) {
                return res.json({'fileid': file_id});
            }
        }
    });

    form.on('error', (err) => {
        errored = true;
        console.error('err!:')
        console.error(err);
        if(err.message && err.message.includes('maxFileSize exceeded')){
            res.statusCode = 413;
            res.json({name: 'Filen fylder for meget', message: ''});
            req.socket.end();
        }
        else{
            res.statusCode = res.statusCode == 200 ? 500 : res.statusCode;
            res.json(err && err.message ? {error: {name: err.name, message: err.message}} : {});
            req.socket.end();
        }
    });
});


router.get('/file/:id', (req, res, next) => {
    const file_id = req.params['id'].replace(/[^a-zA-Z0-9\.]+/g, '');
    const file_path = path.join(config.tmp_path, file_id);

    if(!fs.existsSync(file_path)){
        res.statusCode = 404;
        return next({name: 'Filen findes ikke', message: ''});
    }

    fs.createReadStream(file_path).pipe(res);
});


router.post('/generate/:name', ensureJSON, async (req, res, next) => {

    const template_name = req.params['name'].toLowerCase().replace(/[^a-z]+/g, '');
    if(typeof(req.body.files) === 'object') {
        Object.keys(req.body.files).map(f => {
            req.body.files[f] = req.body.files[f].split('.');
            req.body.files[f][0] = req.body.files[f][0].replace(/[^a-zA-Z0-9]+/g, '');
            req.body.files[f] = config.tmp_path + req.body.files[f].join('.');
        });
    }

    try {
        const data = latexescape(req.body);
        const conf = require(`../templates/${template_name}/conf.js`);
        const rawlatex = await ejs.renderFile(path.join(__dirname, `../templates/${template_name}/${conf.latex}`), data, {async: true});
        const pdf = latex(rawlatex);
        pdf.pipe(res);
    }
    catch(e) {
        console.error(e);
        return next({
            name: 'Kunne ikke generere pdf\'en.',
            message: 'Er du sikker på at du har udfyldt alle felterne korrekt?'
        });
    }
});


module.exports = router;

function autodelete(file_id, timeout = config.autodelete_uploads_interval){
    const file_path = config.tmp_path + file_id;
    setTimeout(() => {
        try{
            fs.unlinkSync(file_path);
            console.log('successfully deleted', file_path);
        }
        catch(e){
            console.log('error deleting file:');
            console.log(e);
            // TODO: log errors silently
        }
    }, timeout);
}
